"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFiatPrice = exports.useMarketPrice = exports.useBondingPricingFromMint = exports.useBondingPricing = exports.useOwnedAmount = exports.useUserOwnedAmount = exports.useSolOwnedAmount = exports.amountAsNum = exports.supplyAsNum = void 0;
const serum_1 = require("@project-serum/serum");
const spl_token_1 = require("@solana/spl-token");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const web3_js_1 = require("@solana/web3.js");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const react_1 = __importStar(require("react"));
const react_async_hook_1 = require("react-async-hook");
const _1 = require(".");
const useAccount_1 = require("./useAccount");
const useAssociatedAccount_1 = require("./useAssociatedAccount");
const useMint_1 = require("./useMint");
const useStrataSdks_1 = require("./useStrataSdks");
const useTokenAccount_1 = require("./useTokenAccount");
const SERUM_PROGRAM_ID = new web3_js_1.PublicKey("9xQeWvG816bUx9EPjHmaT23yvVM2ZWbrrpZb9PusVFin");
const SOL_TOKEN = new web3_js_1.PublicKey("So11111111111111111111111111111111111111112");
// TODO: Use actual connection. But this can't happen in dev
let connection = new web3_js_1.Connection("https://api.mainnet-beta.solana.com");
function supplyAsNum(mint) {
    return amountAsNum(mint.supply, mint);
}
exports.supplyAsNum = supplyAsNum;
function amountAsNum(amount, mint) {
    const decimals = new spl_token_1.u64(Math.pow(10, mint.decimals).toString());
    const decimal = amount.mod(decimals).toNumber() / decimals.toNumber();
    return amount.div(decimals).toNumber() + decimal;
}
exports.amountAsNum = amountAsNum;
function useSolOwnedAmount() {
    const { publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const { info: lamports, loading } = (0, useAccount_1.useAccount)(publicKey || undefined, (_, account) => account.lamports);
    const result = react_1.default.useMemo(() => (lamports || 0) / Math.pow(10, 9), [lamports]);
    return {
        amount: result,
        loading,
    };
}
exports.useSolOwnedAmount = useSolOwnedAmount;
function useUserOwnedAmount(wallet, token) {
    const { associatedAccount } = (0, useAssociatedAccount_1.useAssociatedAccount)(wallet, token);
    const mint = (0, useMint_1.useMint)(token);
    const [amount, setAmount] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        if (mint && associatedAccount) {
            setAmount(amountAsNum(associatedAccount.amount, mint));
        }
    }, [associatedAccount, mint]);
    return amount && Number(amount);
}
exports.useUserOwnedAmount = useUserOwnedAmount;
function useOwnedAmount(token) {
    const { publicKey } = (0, wallet_adapter_react_1.useWallet)();
    return useUserOwnedAmount(publicKey || undefined, token);
}
exports.useOwnedAmount = useOwnedAmount;
/**
 * Get an {@link IPricingCurve} Object that can estimate pricing on this bonding curve,
 * in real time.
 *
 * @param tokenBonding
 * @returns
 */
function useBondingPricing(tokenBonding) {
    const { info: bonding, loading: bondingLoading } = (0, _1.useTokenBonding)(tokenBonding);
    const { info: curve, loading: curveLoading } = (0, _1.useCurve)(bonding === null || bonding === void 0 ? void 0 : bonding.curve);
    const base = (0, useMint_1.useMint)(bonding === null || bonding === void 0 ? void 0 : bonding.baseMint);
    const target = (0, useMint_1.useMint)(bonding === null || bonding === void 0 ? void 0 : bonding.targetMint);
    const { info: baseStorage } = (0, useTokenAccount_1.useTokenAccount)(bonding === null || bonding === void 0 ? void 0 : bonding.baseStorage);
    const pricing = (0, react_1.useMemo)(() => curve &&
        base &&
        target &&
        baseStorage &&
        (0, spl_token_bonding_1.fromCurve)(curve, baseStorage, base, target), [curve, baseStorage, base, target]);
    const loading = (0, react_1.useMemo)(() => curveLoading || bondingLoading, [curveLoading, bondingLoading]);
    return {
        curve: pricing,
        loading,
    };
}
exports.useBondingPricing = useBondingPricing;
/**
 * Same as {@link useBondingPricing}, just from a mint instead of the token bonding key
 *
 * @param mint
 * @param index
 * @returns
 */
function useBondingPricingFromMint(mint, index) {
    const { tokenBondingSdk } = (0, useStrataSdks_1.useStrataSdks)();
    const { result: key, loading } = (0, react_async_hook_1.useAsync)((mint, index) => __awaiter(this, void 0, void 0, function* () { return mint && (tokenBondingSdk === null || tokenBondingSdk === void 0 ? void 0 : tokenBondingSdk.tokenBondingKey(mint, index)); }), [mint, index || 0]);
    const bondingPricing = useBondingPricing(key && key[0]);
    return Object.assign(Object.assign({}, bondingPricing), { loading: bondingPricing.loading || loading });
}
exports.useBondingPricingFromMint = useBondingPricingFromMint;
const useMarketPrice = (marketAddress) => {
    const [price, setPrice] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        const fetch = () => __awaiter(void 0, void 0, void 0, function* () {
            try {
                let market = yield serum_1.Market.load(connection, marketAddress, undefined, SERUM_PROGRAM_ID);
                const book = yield market.loadAsks(connection);
                const top = book.items(false).next().value;
                setPrice(top.price);
            }
            catch (e) {
                console.error(e);
            }
        });
        fetch();
        const interval = setInterval(fetch, 30 * 1000);
        return () => clearInterval(interval);
    }, []);
    return price;
};
exports.useMarketPrice = useMarketPrice;
function useFiatPrice(token) {
    const solPrice = (0, _1.useSolPrice)();
    const { curve } = useBondingPricingFromMint(token);
    const [price, setPrice] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        if ((token === null || token === void 0 ? void 0 : token.toBase58()) == SOL_TOKEN.toBase58()) {
            setPrice(solPrice);
        }
        else {
            // TODO: This is not in sol, so not correct
            setPrice(curve === null || curve === void 0 ? void 0 : curve.current());
        }
    }, [token, solPrice, curve]);
    return price;
}
exports.useFiatPrice = useFiatPrice;
//# sourceMappingURL=bondingPricing.js.map